/* @(#)58  1.7  include/m6_attrmap.h, m6co_headers, sco_3.0, 301FCS 11/18/94 16:44:32, SecureWare, Inc. */

/*
 * Copyright (c) 1990-1994 SecureWare, Inc.  All rights reserved.
 *
 * This Module contains Proprietary Information of SecureWare, Inc.
 * and should be treated as Confidential.
 *
 * Main include file for the MaxSix bitmap translation functions.
 *
 */

/*
 * MaxSix 2.1, July 27, 1992
 * Copyright 1990-1992, SecureWare, Inc.
 */

#ifndef __M6_ATTRMAP__ /*{*/
#define __M6_ATTRMAP__

#include <sys/secdefines.h>

#if SEC_MSIX_INET /*{*/

#define M6AM_EMPTY_TABLE	"NO_MAPPING"

/* The binary table used to drive the translation routines. */
struct attr_x_table {
	struct attr_x_table *next;
	long	total_length;	/* Length of this alloc()'ed data block. */
	short	xlat_type;	/* Translation algorithm to be used. */
	short	exact_match;	/* Does the mapping have to be exact? */
	long	extra_stuff;	/* table specific hack. */
	u_short our_bit_depth;	/* Number of bits in mask on local system. */
	u_short their_bit_depth;/* Number of bits in mask on remote system. */
	u_short	l_for_pos;	/* Length of the forward positive logic table */
	u_short	l_for_neg;	/* Length of the forward negative logic table */
	u_short	l_inv_pos;	/* Length of the inverse positive logic table */
	u_short	l_inv_neg;	/* Length of the inverse negative logic table */
	u_short	*for_pos;	/* Ptr. to the forward positive logic table. */
	u_short	*for_neg;	/* Ptr. to the forward negative logic table. */
	u_short	*inv_pos;	/* Ptr. to the inverse positive logic table. */
	u_short	*inv_neg;	/* Ptr. to the inverse negative logic table. */
};

/*
 * Translation types.
 */
#define M6AM_LIST	0
#define M6AM_LOOKUP	1
#define M6AM_DIRECT	2
#define M6AM_COMPLEX	3

/* 
 * Flags for the exact_match field.
 */
#define M6AM_EXACT_MATCH		1
#define M6AM_BACKWARDS_COMPATIBLE	2

/*
 * Constants embedded within the tables to control logical relationships.
 * We arbitrarily reserve the values 0xfff0 to 0xffff for use by the mapping
 * tables.  This means that we will not be able to handle the mapping of
 * these values.
 */
#define	M6AM_NOENTRY	(u_short)0xffff
#define	M6AM_POSITIVE	(u_short)0xfffe
#define	M6AM_NEGATIVE	(u_short)0xfffd
#define	M6AM_ERROR	(u_short)0xfff2
#define	M6AM_REMOVE	(u_short)0xfff1
#define	M6AM_APPEND	(u_short)0xfff0
#define	M6AM_MAXATTR	(u_short)0xfff0


/* 
 * The direction in which to make the translation.  Forward is from the
 * "local" to "remote" attribute definition.  Inverse is from remote to local.
 */
#define M6AM_FORWARD	0
#define M6AM_INVERSE	1


#ifndef M6AM_NAME_LENGTH
#define M6AM_NAME_LENGTH	48

#if SEC_ACL
struct acl_handler {
	char	*name;
	int	weight;
	int	(*er_to_ir_handler)();
	int	(*ir_to_er_handler)();
};
extern struct acl_handler acl_handlers[];
#endif

/*
 * This structure controls the translation of all attribute type between the
 * local system's format and some other "domain's" format, whatever a 
 * "domain" may be.  This version contains name mappings and is only used
 * in user space.
 */
struct domain_table {
	struct domain_table *next;
	long	total_length;
	short	type;		
	u_short	flags;
	u_short	weight;
	u_long	number;
	u_long	extra_stuff;
	u_long	aliases;
#if SEC_MAC
	struct attr_disk_format *class;
	struct attr_disk_format *cat;
#endif
#if SEC_ILB
	struct attr_disk_format *mark;
#endif
#if SEC_NCAV
	struct attr_disk_format *ncav;
#endif
	struct attr_disk_format *privs;
	struct attr_disk_format *uids;
	struct attr_disk_format *gids;
	struct attr_disk_format *extra1;
	struct attr_disk_format *extra2;
#if SEC_ACL
	struct acl_handler *acl;
#endif
	char name[M6AM_NAME_LENGTH];	/* Name associated with this domain. */
};
#endif


/*
 * Domain types.
 */
#define M6DOM_NULL	0
#define M6DOM_TOKMAP	1
#define M6DOM_CIPSO	2
#define M6DOM_RIPSO	3

/*
 * Domain flags.
 * The values are mutually exclusive.
 */
#define M6AM_BINARY	1
#define M6AM_ASCII	2
#define M6AM_BOTH	(M6AM_ASCII + M6AM_BINARY)

int m6am_errno;			/* Error return for function calls. */
#define TE_ERRNO		1
#define	TE_STR_TO_BITS		2
#define	TE_BITS_TO_STR		3
#define	TE_BITS_TO_BITS		4

#define	M6AM_FILE_ASCII		"/tcb/files/M6MAPPINGS"
#define	M6AM_FILE_BINARY	"/tcb/files/M6BINMAP"
#define	M6AM_DOMAINS		"/tcb/files/M6DOMAINS"

/*
 * The on disk format of the name/ID mappings.
 */
struct identifier {
        u_short value;		/* Bit value of identifier. */
	u_short	sort;		/* Field used to alphabetize the ids. */
        char    *name;		/* Pointer to name of id. */
};
#define M6AM_LOCAL_NAMES	0
#define M6AM_REMOTE_NAMES	1

/* 
 * The on disk binary format of an attribute mapping table 
 */
struct attr_disk_format {
	struct	attr_disk_format *next;
	char 	name[M6AM_NAME_LENGTH];	/* Name associated with this table. */
	long 	total_length;		/* Total length of all structures. */
	u_short	l_name_sz;		/* Number of entries in l_name table */
	u_short	r_name_sz;		/* Number of entries in r_name table */
	struct 	identifier *l_names;	/* Pointer to l_name table */
	struct 	identifier *r_names;	/* Pointer to r_name table */
	struct 	attr_x_table table;	/* Translation table. */
};

/*
 * Defines used by the parser
 */
#ifndef MAX_NAME_TABLE_SIZE
#define MAX_NAME_TABLE_SIZE	2000
#endif
#ifndef MAX_RULE_TABLE_SIZE
#define MAX_RULE_TABLE_SIZE	2000
#endif
#ifndef MAX_RULE_ELEMENTS
#define MAX_RULE_ELEMENTS	100
#endif
#ifndef M6AM_LOOKUP_THRESHOLD
#define M6AM_LOOKUP_THRESHOLD	16
#endif
#ifndef M6AM_LOOKUP_PTHRESHOLD
#define M6AM_LOOKUP_PTHRESHOLD	1/2
#endif
#ifndef MAX_COMPOUND_NAMES
#define MAX_COMPOUND_NAMES	3
#endif
typedef struct identifier name_table_t[MAX_NAME_TABLE_SIZE];
typedef u_short rule_table_t[MAX_RULE_TABLE_SIZE*2];

#if !defined(__hpxt) && !defined(TEKXP)
#define bcopy(ff, tt, cc)       memcpy((tt), (ff), (cc))
#define bzero(ff, cc)           memset((ff), 0, (cc))
#endif

#endif /* SEC_MSIX_INET }*/
#endif /* __M6_ATTRMAP__ }*/
